#! /usr/bin/python3
# Creates Texinfo documentation from the source

import re
import sys

print("""\
@c Generated from %s by get-commands.py
@c Do not modify!

@table @asis""" % sys.argv[1])
for line in open(sys.argv[1], 'r'):
    m = re.match(r'^\s*UNIMPL_CMD\s*\(\s*"([^"]*)"\s*,\s*"([^"]*)"\)\s*$', line)
    if m:
        command, description = m.groups()
        print("@item @cmd{%s}\n%s\n" % (command, description))
print("""\
@end table
@c Local Variables:
@c buffer-read-only: t
@c End:""")




#! /usr/bin/env python
# Copyright (C) 2020  Free Software Foundation

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys

fmt_name = sys.argv[1]
templates = sys.argv[2:]

times = (#  D  HH  MM     SS
         (  0,  0,  0,  0.00),
         (  1,  4, 50, 38.68),
         (  5, 12, 31, 35.82),
         (  0, 12, 47, 53.41),
         (  3,  1, 26,  0.69),
         (  1, 20, 58, 11.19),
         ( 12,  7, 36,  5.98),
         ( 52, 15, 43, 49.27),
         (  7,  4, 25,  9.24),
         (  0,  6, 49, 27.89),
         ( 20,  2, 57, 52.56),
         (555, 16, 45, 44.12),
         (120, 21, 30, 57.27),
         (  0,  4, 25,  9.98),
         (  3,  6, 49, 27.24),
         (  5,  2, 57, 52.13),
         (  0, 16, 45, 44.35),
         (  1, 21, 30, 57.32),
         ( 10, 22, 30,  4.27),
         ( 22,  1, 56, 51.18))

syntax_file = open('%s.sps' % fmt_name, 'w')
syntax_file.write('''\
DATA LIST NOTABLE FILE='%(fmt_name)s.input'/%(fmt_name)s 1-40 (%(fmt_name)s).
PRINT OUTFILE='%(fmt_name)s.output'/%(fmt_name)s (F16.2).
EXECUTE.
''' % {'fmt_name': fmt_name})
syntax_file.close()

expout_file = open('expout', 'w')
input_file = open('%s.input' % fmt_name, 'w')

def print_all_formats(d, h, m, s, template, formatted, expected, sign):
    if template != '':
        c = template[0]
        template = template[1:]
        if c == '+':
            assert sign == ''
            for new_sign in ('', '-', '+'):
                print_all_formats(d, h, m, s, template,
                                  formatted + new_sign, expected, new_sign)
        elif c == 'D':
            for f in ('%d', '%02d'):
                print_all_formats(0, h, m, s, template,
                                  formatted + (f % d), expected + d * 86400,
                                  sign)
        elif c == 'H':
            for f in ('%d', '%02d'):
                print_all_formats(0, 0, m, s, template,
                                  formatted + (f % (h + d * 24)),
                                  expected + h * 3600 + d * 86400,
                                  sign)
        elif c == 'M':
            for f in ('%d', '%02d'):
                print_all_formats(0, 0, 0, s, template,
                                  formatted + (f % (m + h * 60 + d * 1440)),
                                  expected + m * 60 + h * 3600 + d * 86400,
                                  sign)
        elif c == 'S':
            for f in ('%.0f', '%02.0f', '%.1f', '%.2f'):
                ns = s + m * 60 + h * 3600 + d * 86400
                formatted_s = f % ns
                print_all_formats(0, 0, 0, 0, template,
                                  formatted + formatted_s,
                                  expected + float(formatted_s),
                                  sign)
        elif c == ':':
            for f in (' ', ':'):
                print_all_formats(d, h, m, s, template, formatted + f,
                                  expected, sign)
        elif c == ' ':
            print_all_formats(d, h, m, s, template, formatted + ' ',
                              expected, sign)
        else:
            assert False
    else:
        # Write the formatted value to fmt_name.input.
        input_file.write('%s\n' % formatted)

        # Write the expected value to 'expout'.
        if sign == '-' and expected > 0:
            expected = -expected
        expected_s = '%17.2f\n' % expected
        expected_s = expected_s.replace(' 0.', '  .')
        expout_file.write(expected_s)


for template in templates:
    for time in times:
        d, h, m, s = time
        print_all_formats(d, h, m, s, template, '', 0, '')




#! /usr/bin/python3

# Copyright (C) 2020, 2021 Free Software Foundation
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import sys

def process_converter(fields):
    if not fields or fields[0] == '{':
        return

    global codepages
    cps = {}
    iana = []
    other = []

    i = 0
    while i < len(fields):
        name = fields[i]
        i += 1

        if i < len(fields) and fields[i] == '{':
            i += 1

            standards = set()
            while True:
                standard = fields[i]
                i += 1
                if standard == '}':
                    break
                standards.add(standard)
            if 'IANA*' in standards:
                iana = [name] + iana
            elif 'IANA' in standards:
                iana += [name]
            elif any(map(lambda s: s.endswith('*'), standards)):
                other = [name] + other
            else:
                other += [name]
        else:
            # Untagged names are completely nonstandard.
            continue

        m = re.match(r'cp([0-9]+)$', name)
        if m:
            cps[CP] = int(m.group(1))
            continue

        m = re.match(r'windows-([0-9]+)$', name)
        if m:
            cps[WINDOWS] = int(m.group(1))
            continue

        m = re.match(r'ibm-([0-9]+)$', name)
        if m:
            cps[IBM] = int(m.group(1))
            continue

    # If there are no tagged names then this is completely nonstandard.
    if not iana and not other:
        return

    for cp in cps.keys():
        codepages.setdefault(cps[cp], {})[cp] = iana + other

if len(sys.argv) != 2 or sys.argv[1] == '--help':
    sys.stderr.write("""\
%s: generate code page tables from ICU encoding list
usage: %s CONVRTRS-TXT > sys-file-encoding.c

To update the encoding data, get the latest ICU encoding data from:
https://raw.githubusercontent.com/unicode-org/icu/\
main/icu4c/source/data/mappings/convrtrs.txt
""" % (sys.argv[0], sys.argv[0]))
    sys.exit(0 if len(sys.argv) == 2 and sys.argv[1] == '--help' else 1)

WINDOWS = 3             # Windows code pages.
IBM = 2                 # IBM code pages.
CP = 1                  # Java (?) code pages.
sources = { WINDOWS: "windows", IBM: "ibm", CP: "cp" }

codepages = {}

converter = []
for line in open(sys.argv[1], 'r'):
    line = line.rstrip()
    comment_ofs = line.find('#')
    if comment_ofs >= 0:
        line = line[:comment_ofs]
    if line.lstrip() == line:
        process_converter(converter)
        converter = []
    converter += line.split()
process_converter(converter)

print("""\
/* -*- mode: c; buffer-read-only: t -*-

   Generated by sys-file-encoding.py.  Do not modify!
*/

/*
PSPP - a program for statistical analysis.
Copyright (C) 2017 Free Software Foundation, Inc.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <config.h>

#include "data/sys-file-private.h"

struct sys_encoding sys_codepage_number_to_name[] = {""")

for cpnumber, value in sorted(codepages.items()):
    source = max(value.keys())
    name = value[source][0]
    print('  { %s, "%s" },' % (cpnumber, name))
print("""  { 0, NULL }
};
""")

names = {}
for cpnumber, value in sorted(codepages.items()):
    for source, value2 in value.items():
        for name in value2:
            names.setdefault(name, {}).setdefault(source, []).append(cpnumber)

print('struct sys_encoding sys_codepage_name_to_number[] = {')
for name in sorted(names.keys()):
    for source in sorted(sources.keys(), reverse=True):
        if source not in names[name]:
            continue

        numbers = names[name][source]

        # The only two encodings that currently print this are KSC_5601
        # and KS_C_5601-1987, for code pages 949 and 51949.  It looks to
        # me like the correct code page number is 949, which is the one
        # chosen (because the numbers are in sorted order).
        if len(numbers) > 1:
            print('  /* %s has multiple numbers for %s: %s */'
                  % (name, sources[source], ' '.join(map(str, numbers))))
        print('  { %s, "%s" },' % (numbers[0], name))
        break
print("""\
  { 0, NULL }
};""")

